
# *******************************************************************************************
# SharpCap script "INTI_SharpCap_menu.py"
# Jean-Francois
#
# Version: 3.3      : 2023/01/04    Calculation of the optimal telescope rate
# Version: 3.2      : 2022/10/16    Menu version and calculation of the sun disk angle
# Version: 3.1      : 2022/10/02    Change name format for Pro-Am collaboration
# Version: 3.0      : 2022/09/18    Add scan information in SER film
# Version: 2.3      : 2022/07/29    Change to the script from TheSmiths v3.5
# Version: 2.2      : 2021/11/18    Use SHG_MAIN-v8
# Version: 2.1      : 2021/08/09    Add SER parameters
# Version: 2.0      : 2021/08/01
#
# Conditions to be fullfilled before starting the script with SharpCap:
# ------------------------------------------------------------------
# 1. Camera is selected and running
# 2. All the settings of the camera are selected (ROI, binning, exposure time, gain, ...)
# 3. The output file format SER is selected
# 4. Files location is set in SharpCap
# 5. The telescope is connected (if the telescope is motorised)
#
# Conditions to be fullfilled before use of the script:
# ------------------------------------------------------------------
# 1. Python 3.7 or above shall be installed on the computer
# 2. The variable PATH shall be set to the location of the Python installation (should be done automatically)
# 3. For test ... start Inti_recon.py or SHG_MAIN.py for checking the film processing
#
#
# Description of the script version 3.3:
# --------------------------------------
# New version with buton from the calculation of the telescope rate for Sx/Sy = 1.
# The user has to set the telescope focal length.
# The script start a capture of 2 seconds for the real frame per second (the film is deleted after use).
# The optimal telescope rate is shown in the Process output window.
# Note: the actual version of the TheSmith script is the 3.7
# https://github.com/thelondonsmiths/Solex_ser_recon_EN/releases/tag/v3.7_Windows
#
# Description of the script version 3.2:
# --------------------------------------
# New version with buton from the menu bar of SharpCap.
# Calculation of the Carrington angle of the sun disk orientation.
# Correction of some bugs. Add the output window for the information.
#
# Description of the script version 3.1:
# --------------------------------------
# Add rename SER file to Pro-Am format.
#
# Description of the script version 3.0: 
# --------------------------------------
# The scan direction and the mount side information are saved in the SER file.
# These informations are used in the INTI software for the sun disc image rotation.
#
# Description of the script version 2.3: 
# --------------------------------------
# Update of the script with the version v3.5 of the Python script of theSmiths
# https://github.com/thelondonsmiths/Solex_ser_recon_EN/releases/tag/v3.5
#
# Description of the script version 2.0: 
# --------------------------------------
# The script starts the saving of a SER film until the user press the STOP button.
# At the end of the film, the script starts the INTI processing, if selected.
# Depending on the selection, the image processing is shown on the monitor,
# and the selected files (PNG and/or FITS) are saved on the computer. 
# The script commands the movement of the telescope and starts/stops the film saving.
#
#
# *******************************************************************************************

import os
import time
import datetime
import math
import clr
clr.AddReference("System.Windows.Forms")
clr.AddReference("System.Drawing")
clr.AddReference("ASCOM.DriverAccess")
clr.AddReference("ASCOM.Astrometry")        # ASCOM AstroUtils to get current JulianDate

from System.Windows.Forms import *
from System.Drawing import Pen, Brush, Point, Color, Font, FontStyle, Size, Image, Icon, Rectangle, Brushes, SolidBrush, Pens
from ASCOM.DriverAccess import *
from ASCOM.Astrometry import *

# ****************************************************************************************************
# ****************************************************************************************************
#
# User information, only maximum 40 characteres long (it will be cut automatically to 40)
Observer = "Observer"
Telescope = "Spectro, telescope, camera"
Identificateur = "_12345AB"             # The suffix for the file name for the Pro-Am collaboration project
#
# Note the field "Telescope" in the SER file is used for the scanning information to INTI software
#
# ****************************************************************************************************
# ****************************************************************************************************

tracking = SharpCap.Mounts.SelectedMount.Tracking       # Is the telescope tracking ?
Tel_Film = "T"                                          # Telescope movement mode at start

# User to comment the lines 669 and 774     # file = "C:\\Temp\\10_28_39.ser"
# or change the file path and file name with an existing file for testing the script.

# User to modify the path of the python script. Note the double "\\".
command_python = "python.exe D:\\Software_Installation\\Astro-Software\\INTI\\solex_ser_recon_EN_v3.7\\SHG_MAIN.py"
# User to modify the path of the icone. Note the double "\\".
default_icon = "D:\\Scripts\\INTI_SharpCap\\SHG.ico"


def Carrington(now_UTC):
    now_JD = AstroUtils.AstroUtils().JulianDateUtc
    theta = (now_JD - 2398220) * 360/25.38
    I = 7.25
    K = 73.6667 + 1.3958333*(now_JD - 2396758)/36525
    T = (now_JD - 2451545)/36525
    Lo = (0.0003032*T + 36000.76983)*T + 280.46645
    M = ((-0.00000048*T - 0.0001559)*T + 35999.05030)*T + 357.52910
    C = ((-0.000014*T - 0.004817)*T + 1.914600)*math.sin(math.radians(M))
    C = C +(-0.000101*T - 0.019993)*math.sin(math.radians(2*M)) + 0.000290*math.sin(math.radians(3*M))
    S_true_long = Lo + C
    Lambda = S_true_long - 0.00569 - 0.00478*math.sin(math.radians(125.04 - 1934.136*T))
    Lambda_cor = Lambda + 0.004419
    x = math.degrees(math.atan(-math.cos(math.radians(Lambda_cor)) * math.tan(math.radians(23.440144))))
    y = math.degrees(math.atan(-math.cos(math.radians(Lambda - K)) * math.tan(math.radians(I))))
    P = x + y
    Bo = math.degrees(math.asin(math.sin(math.radians(Lambda - K)) * math.sin(math.radians(I))))
    return(str(round(P,2)),str(round(Bo,2)))


class StatePanel(Panel):
    def __init__(self, label, width, height):
        super(StatePanel, self).__init__()
        self.Width = width
        self.Height = height
        label_widget = Label()
        label_widget.Text = label
        label_widget.Dock = DockStyle.Top
        label_widget.Height = 25
        label_widget.BackColor = Color.LightGray
        label_widget.Font = Font(label_widget.Font, FontStyle.Bold)
        label_widget.Padding = Padding(3)

        state_widget = RichTextBox()
        state_widget.Dock = DockStyle.Top
        state_widget.Height = height - label_widget.Height
        state_widget.Multiline = True
        state_widget.ScrollBars = RichTextBoxScrollBars.Vertical
        state_widget.WordWrap = True
        state_widget.ReadOnly = True

        self.Controls.Add(state_widget)
        self.Controls.Add(label_widget)
        self.label = label_widget
        self.info = state_widget

    def log(self, text, error=False):
        start_selection = self.info.Text.Length
        self.info.ReadOnly = False
        self.info.AppendText(str(text)+"\n")
        self.info.ReadOnly = True
        if error:
            self.info.SelectionStart = start_selection
            self.info.SelectionLength = self.info.Text.Length - start_selection
            self.info.SelectionColor = Color.Red
        self.info.SelectionStart = self.info.Text.Length
        self.info.ScrollToCaret()

    def log_status(self, text):
        self.label.Text = (str(text))

    def log_error(self, text):
        self.log(text, True)

    def clear_log(self):
        self.info.ReadOnly = False
        self.info.Text = ""
        self.info.ReadOnly = True

class SHG_CaptureMenuForm(Form):
    def __init__(self):
        self.Text = "SHG and processing with INTI"
        self.Width = 1004
        self.Height = 380
        #self.CenterToScreen()
        self.setupCheckButtons()

    def setupCheckButtons(self): 
        self.checkbox_recon = CheckBox()
        self.checkbox_recon.Text = "Process with INTI"
        self.checkbox_recon.Location = Point(15, 10)
        self.checkbox_recon.AutoSize = True
        self.checkbox_recon.CheckedChanged += self.CheckBoxChanged

        self.checkbox_showimage = CheckBox()
        self.checkbox_showimage.Text = "Show images"
        self.checkbox_showimage.Location = Point(15, 35)
        self.checkbox_showimage.AutoSize = True
        self.checkbox_showimage.Enabled = False
        self.checkbox_showimage.Checked = True
        self.checkbox_showimage.CheckedChanged += self.CheckBoxChanged

        self.checkbox_saveclahepng = CheckBox()
        self.checkbox_saveclahepng.Text = "Save PNG images"
        self.checkbox_saveclahepng.Location = Point(15, 60)
        self.checkbox_saveclahepng.AutoSize = True
        self.checkbox_saveclahepng.Enabled = False
        self.checkbox_saveclahepng.CheckedChanged += self.CheckBoxChanged

        self.checkbox_savefit = CheckBox()
        self.checkbox_savefit.Text = "Save FITS images"
        self.checkbox_savefit.Location = Point(15, 85)
        self.checkbox_savefit.AutoSize = True
        self.checkbox_savefit.Enabled = False
        self.checkbox_savefit.CheckedChanged += self.CheckBoxChanged

        box_height = 55     # Telescope control button dimension
        box_width = 55
        pos_x = 280         # Telescope control button position
        pos_y = 10
        dist = 5            # Telescope control button gap

        self.DEC_plus_Button = Button()
        self.DEC_plus_Button.Text = "DEC +"
        self.DEC_plus_Button.Font = Font("Times", 12, FontStyle.Bold)
        self.DEC_plus_Button.Location = Point(pos_x, pos_y)
        self.DEC_plus_Button.Height = box_height
        self.DEC_plus_Button.Width = box_width
        self.DEC_plus_Button.Click += self.DEC_plus

        self.RA_plus_Button = Button()
        self.RA_plus_Button.Text = "RA +"
        self.RA_plus_Button.Font = Font("Times", 12, FontStyle.Bold)
        self.RA_plus_Button.Location = Point(pos_x - box_width - dist, pos_y + box_height + dist)
        self.RA_plus_Button.Height = box_height
        self.RA_plus_Button.Width = box_width
        self.RA_plus_Button.Click += self.RA_plus

        self.Center_Button = Button()
        self.Center_Button.Text = "T"
        self.Center_Button.Font = Font("Times", 28, FontStyle.Bold)
        self.Center_Button.Location = Point(pos_x, pos_y + box_height + dist)
        self.Center_Button.Height = box_height
        self.Center_Button.Width = box_width
        self.Center_Button.Click += self.Center

        self.RA_minus_Button = Button()
        self.RA_minus_Button.Text = "RA -"
        self.RA_minus_Button.Font = Font("Times", 12, FontStyle.Bold)
        self.RA_minus_Button.Location = Point(pos_x + box_width + dist, pos_y + box_height + dist)
        self.RA_minus_Button.Height = box_height
        self.RA_minus_Button.Width = box_width
        self.RA_minus_Button.Click += self.RA_minus

        self.DEC_minus_Button = Button()
        self.DEC_minus_Button.Text = "DEC -"
        self.DEC_minus_Button.Font = Font("Times", 12, FontStyle.Bold)
        self.DEC_minus_Button.Location = Point(pos_x, pos_y + 2*box_height + 2*dist)
        self.DEC_minus_Button.Height = box_height
        self.DEC_minus_Button.Width = box_width
        self.DEC_minus_Button.Click += self.DEC_minus

        self.CaptureButton = Button()
        self.CaptureButton.Text = "Capture"
        self.CaptureButton.Location = Point(10, 285)
        self.CaptureButton.Width = 110
        self.CaptureButton.AutoSize = True
        self.CaptureButton.Click += self.capture

        self.StopButton = Button()
        self.StopButton.Text = "Stop"
        self.StopButton.Location = Point(125, 285)
        self.StopButton.Width = 110
        self.StopButton.AutoSize = True
        self.StopButton.Click += self.stop

        self.ExitButton = Button()
        self.ExitButton.Text = "Exit"
        self.ExitButton.Location = Point(280, 285)
        self.ExitButton.Width = 100
        self.ExitButton.AutoSize = True
        self.ExitButton.Click += self.exit

        self.FrameButton = Button()
        self.FrameButton.Text = "Frame Rate"
        self.FrameButton.Location = Point(410, 285)
        self.FrameButton.Width = 100
        self.FrameButton.AutoSize = True
        self.FrameButton.Click += self.framerate

        self.TelFocalTextLabel = Label()
        self.TelFocalTextLabel.Text = "Telescope Focale [mm]:"
        self.TelFocalTextLabel.Location = Point(530, 290)
        self.TelFocalTextLabel.AutoSize = True

        self.TelFocaleTextbox = TextBox()
        self.TelFocaleTextbox.Text = "432"
        self.TelFocaleTextbox.Location = Point(740, 285)
        self.TelFocaleTextbox.Width = 60

        self.TelRateLabel = Label()
        self.TelRateLabel.Text = "Telescope Rate:"
        self.TelRateLabel.Location = Point(15, 115)
        self.TelRateLabel.AutoSize = True

        self.TelRateTextbox = TextBox()
        self.TelRateTextbox.Text = "16"
        self.TelRateTextbox.Location = Point(145, 112)
        self.TelRateTextbox.Width = 30

        self.checkbox_disc = CheckBox()
        self.checkbox_disc.Text = "Full sun disc"
        self.checkbox_disc.Location = Point(15, 145)
        self.checkbox_disc.AutoSize = True
        self.checkbox_disc.CheckedChanged += self.CheckBoxChanged

        self.checkbox_rename = CheckBox()
        self.checkbox_rename.Text = "Rename SER film (UTC)"
        self.checkbox_rename.Location = Point(15, 170)
        self.checkbox_rename.AutoSize = True
        self.checkbox_rename.CheckedChanged += self.CheckBoxChanged

        self.InformationLabel = Label()
        self.InformationLabel.Text = "Film filename info. :"
        self.InformationLabel.Location = Point(15, 200)
        self.InformationLabel.Width = 115  

        self.InformationTextbox = TextBox()
        self.InformationTextbox.Text = ""
        self.InformationTextbox.Location = Point(130, 200)
        self.InformationTextbox.Width = 150

        self.InfObserverLabel = Label()
        self.InfObserverLabel.Text = "SER Info. Observer:"
        self.InfObserverLabel.Location = Point(15, 230)
        self.InfObserverLabel.Width = 115  
        
        self.InfObserverTextbox = TextBox()
        self.InfObserverTextbox.Text = Observer
        self.InfObserverTextbox.Location = Point(130, 230)
        self.InfObserverTextbox.Width = 250

        self.InfTelescopeLabel = Label()
        self.InfTelescopeLabel.Text = "SER Info. Telescope:"
        self.InfTelescopeLabel.Location = Point(15, 255)
        self.InfTelescopeLabel.Width = 115 

        self.InfTelescopeTextbox = TextBox()
        self.InfTelescopeTextbox.Text = Telescope
        self.InfTelescopeTextbox.Location = Point(130, 255)
        self.InfTelescopeTextbox.Width = 250

        self.process = StatePanel("Process output", width = 560, height = 270)
        self.process.Location = Point(410, 10)
        self.process.BackColor   = Color.White
        self.process.BorderStyle = BorderStyle.FixedSingle

        self.AcceptButton = self.CaptureButton
        self.AcceptButton = self.StopButton
        self.CancelButton = self.ExitButton

        self.Controls.Add(self.checkbox_recon)
        self.Controls.Add(self.checkbox_showimage)
        self.Controls.Add(self.checkbox_saveclahepng)
        self.Controls.Add(self.checkbox_savefit)
        self.Controls.Add(self.checkbox_disc)
        self.Controls.Add(self.checkbox_rename)

        self.Controls.Add(self.DEC_plus_Button)
        self.Controls.Add(self.RA_minus_Button)
        self.Controls.Add(self.Center_Button)
        self.Controls.Add(self.RA_plus_Button)
        self.Controls.Add(self.DEC_minus_Button)

        self.Controls.Add(self.CaptureButton)
        self.Controls.Add(self.StopButton)
        self.Controls.Add(self.ExitButton)
        self.Controls.Add(self.FrameButton)
        self.Controls.Add(self.TelFocalTextLabel)
        self.Controls.Add(self.TelFocaleTextbox)

        self.Controls.Add(self.TelRateLabel)
        self.Controls.Add(self.TelRateTextbox)
        self.Controls.Add(self.InformationLabel)
        self.Controls.Add(self.InformationTextbox)
        self.Controls.Add(self.InfObserverLabel)
        self.Controls.Add(self.InfObserverTextbox)
        self.Controls.Add(self.InfTelescopeLabel)
        self.Controls.Add(self.InfTelescopeTextbox)

        self.Controls.Add(self.process)
        self.processing = False

        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        log("Telescope control: " + str(SharpCap.Mounts.SelectedMount))
        if (SharpCap.Mounts.SelectedMount.Connected):
            log("The telescope is connected")
        else:
            log("The telescope is not connected.")
        if (SharpCap.Mounts.SelectedMount.Tracking):
            log("The telescope is tracking")
        else:
            log("The tracking of the telescope is off")

    def CheckBoxChanged(self, sender, args):
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        if (self.checkbox_recon.Checked):
            flag_recons = True
            self.checkbox_showimage.Enabled = True
            self.checkbox_saveclahepng.Enabled = True
            self.checkbox_savefit.Enabled = True
        else:
            flag_recons = False
            self.checkbox_showimage.Enabled = False
            self.checkbox_saveclahepng.Enabled = False
            self.checkbox_savefit.Enabled = False

        if (self.checkbox_showimage.Checked):
            flag_showimages = True
        else:
            flag_showimages = False

        if (self.checkbox_saveclahepng.Checked):
            flag_saveclahepng = True
        else:
            flag_saveclahepng = False

        if (self.checkbox_savefit.Checked):
            flag_saveFITS = True
        else:
            flag_saveFITS = False

        if (self.checkbox_disc.Checked):
            flag_sundisc = True
        else:
            flag_sundisc = False

        if (self.checkbox_rename.Checked):
            flag_rename = True
        else:
            flag_rename = False

    def DEC_plus(self, sender, event):      # Action by clicking on the DEC+ button
        global tracking
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        self.DEC_plus_Button.BackColor  = Color.Gray
        self.RA_minus_Button.BackColor  = Color.LightGray
        self.RA_plus_Button.BackColor   = Color.LightGray
        self.DEC_minus_Button.BackColor = Color.LightGray
        if (SharpCap.Mounts.SelectedMount.Connected):
            tracking = SharpCap.Mounts.SelectedMount.Tracking
            rate = int(self.TelRateTextbox.Text)
            log("DEC+ movement with rate = " + str(rate))
            SharpCap.Mounts.SelectedMount.MoveAxis(1, rate)
            self.Center_Button.Text = "STOP"
            self.Center_Button.Font = Font("Times", 9, FontStyle.Bold)
            self.Center_Button.BackColor = Color.Red
            if (Tel_Film == "C"):
                log("Start capture SER film")
                SharpCap.SelectedCamera.PrepareToCapture()
                SharpCap.SelectedCamera.RunCapture()
        else:
            log("Telescope is not connected")

    def RA_minus(self, sender, event):      # Action by clicking on the RA- button
        global tracking
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        self.DEC_plus_Button.BackColor  = Color.LightGray
        self.RA_minus_Button.BackColor  = Color.Gray
        self.RA_plus_Button.BackColor   = Color.LightGray
        self.DEC_minus_Button.BackColor = Color.LightGray
        if (SharpCap.Mounts.SelectedMount.Connected):
            tracking = SharpCap.Mounts.SelectedMount.Tracking
            rate = int(self.TelRateTextbox.Text)
            log("RA-  movement with rate = " + str(rate))
            SharpCap.Mounts.SelectedMount.MoveAxis(0, rate + 1)
            self.Center_Button.Text = "STOP"
            self.Center_Button.Font = Font("Times", 9, FontStyle.Bold)
            self.Center_Button.BackColor = Color.Red
            if (Tel_Film == "C"):
                log("Start capture SER film")
                SharpCap.SelectedCamera.PrepareToCapture()
                SharpCap.SelectedCamera.RunCapture()
        else:
            log("Telescope is not connected")

    def RA_plus(self, sender, event):      # Action by clicking on the RA+ button
        global tracking
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        self.DEC_plus_Button.BackColor  = Color.LightGray
        self.RA_minus_Button.BackColor  = Color.LightGray
        self.RA_plus_Button.BackColor   = Color.Gray
        self.DEC_minus_Button.BackColor = Color.LightGray
        if (SharpCap.Mounts.SelectedMount.Connected):
            tracking = SharpCap.Mounts.SelectedMount.Tracking
            rate = int(self.TelRateTextbox.Text)
            if (rate != 1):
                SharpCap.Mounts.SelectedMount.MoveAxis(0, -rate + 1)
            else:
                SharpCap.Mounts.SelectedMount.MoveAxis(0, -rate)
            log("RA+  movement with rate = " + str(rate))
            self.Center_Button.Text = "STOP"
            self.Center_Button.Font = Font("Times", 9, FontStyle.Bold)
            self.Center_Button.BackColor = Color.Red

            if (Tel_Film == "C"):
                log("Start capture SER film")
                SharpCap.SelectedCamera.PrepareToCapture()
                SharpCap.SelectedCamera.RunCapture()
        else:
            log("Telescope is not connected")

    def DEC_minus(self, sender, event):      # Action by clicking on the DEC- button
        global tracking
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        self.DEC_plus_Button.BackColor  = Color.LightGray
        self.RA_minus_Button.BackColor  = Color.LightGray
        self.RA_plus_Button.BackColor   = Color.LightGray
        self.DEC_minus_Button.BackColor = Color.Gray
        if (SharpCap.Mounts.SelectedMount.Connected):
            tracking = SharpCap.Mounts.SelectedMount.Tracking
            rate = int(self.TelRateTextbox.Text)
            log("DEC- movement with rate = " + str(rate))
            SharpCap.Mounts.SelectedMount.MoveAxis(1, -rate)
            self.Center_Button.Text = "STOP"
            self.Center_Button.Font = Font("Times", 9, FontStyle.Bold)
            self.Center_Button.BackColor = Color.Red
            if (Tel_Film == "C"):
                log("Start capture SER film")
                SharpCap.SelectedCamera.PrepareToCapture()
                SharpCap.SelectedCamera.RunCapture()
        else:
            log("Telescope is not connected")

    def Center(self, sender, event):      # Action by clicking on the center button
        global Tel_Film, tracking, command_line
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        if (self.Center_Button.Text == "STOP"):
            self.Center_Button.Text = Tel_Film
            if (Tel_Film == "T"):
                log("Stop telescope movement")
        elif (self.Center_Button.Text == "T"):
            self.Center_Button.Text = "C"
            self.Center_Button.BackColor = Color.LightGreen
            Tel_Film = "C"
            log("Change to camera capture mode")
        elif (self.Center_Button.Text == "C"):
            self.Center_Button.Text = "T"
            self.Center_Button.BackColor = Color.LightGray
            Tel_Film = "T"
            log("Change to telescope movement mode")

        if (SharpCap.Mounts.SelectedMount.Connected):
            if (SharpCap.Mounts.SelectedMount.Slewing):
                SharpCap.Mounts.SelectedMount.Stop()
                SharpCap.Mounts.SelectedMount.Tracking = tracking
                self.Center_Button.Text = Tel_Film
                self.Center_Button.Font = Font("Times", 28, FontStyle.Bold)
                if (Tel_Film == "C"):
                    self.Center_Button.BackColor = Color.LightGreen
                else:
                    self.Center_Button.BackColor = Color.LightGray
        else:
            log("Telescope is not connected")

        if (SharpCap.SelectedCamera.Capturing):
            log("Stop capture SER film")
            SharpCap.SelectedCamera.StopCapture()
            SharpCap.ShowNotification(None)

            if (self.DEC_plus_Button.BackColor == Color.Gray):
                Tel_Scan = "DECP"
            if (self.RA_minus_Button.BackColor == Color.Gray):
                Tel_Scan = "RAM"
            if (self.RA_plus_Button.BackColor == Color.Gray):
                Tel_Scan = "RAP"
            if (self.DEC_minus_Button.BackColor == Color.Gray):
                Tel_Scan = "DECM"

            Tel_Side = str(SharpCap.Mounts.SelectedMount.SideOfPier)
            if (Tel_Side == "WestSideLookingEast"):
                Tel_Side = "East"
            if (Tel_Side == "EastSideLookingWest"):
                Tel_Side = "West"
            if (self.checkbox_disc.Checked):
                Sun_Disc = "FullDisc"
            else:
                Sun_Disc = "PartialDisc"

            P, Bo = Carrington(datetime.datetime.utcnow())
            log(Tel_Scan + " , " + Tel_Side + " , " + Sun_Disc + " , P = " + P + " , Bo = " + Bo)

            file = SharpCap.GetLastCaptureFilename()

            f=open(file, "r+")
            f.seek(42)
            t_observer = self.InfObserverTextbox.Text
            t_observer = t_observer[:40]
            t_observer = t_observer.ljust(40)

            t_Instrument = Tel_Scan + " " + Tel_Side + " " + Sun_Disc + " " + self.TelRateTextbox.Text + " " + P + " " + Bo
            t_Instrument = t_Instrument[:40]
            t_Instrument = t_Instrument.ljust(40)

            t_Telescope = self.InfTelescopeTextbox.Text
            t_Telescope = t_Telescope[:40]
            t_Telescope = t_Telescope.ljust(40)

            f.write(t_observer)
            f.write(t_Instrument)
            f.write(t_Telescope)
            f.close()

            if (self.checkbox_rename.Checked):
                log("Rename SER file")
                current_time = datetime.datetime.utcnow()
                if (self.InformationTextbox.Text == ""):
                    newname = os.path.dirname(file) +"\\" + current_time.strftime('%Y%m%d%H%M%S') + Identificateur + ".ser"
                else:
                    newname = os.path.dirname(file) +"\\" + current_time.strftime('%Y%m%d%H%M%S') + Identificateur + "_" + self.InformationTextbox.Text + ".ser"
                log(newname)
                os.rename(file, newname)
                try:
                    filename_settings = os.path.splitext(file)[0] + ".CameraSettings.txt"
                    newfilename_settings = os.path.splitext(newname)[0] + ".CameraSettings.txt"
                    os.rename(filename_settings, newfilename_settings)
                except:
                    log("CameraSettings.txt file does not exist")
                file = newname
            else:
                if (self.InformationTextbox.Text != ""):
                    filename = os.path.splitext(file)[0]
                    newname = filename + "_" + self.InformationTextbox.Text + ".ser"
                    log(newname)
                    os.rename(file, newname)
                    try:
                        filename_settings = os.path.splitext(file)[0] + ".CameraSettings.txt"
                        newfilename_settings = os.path.splitext(file)[0] +  "_" + self.InformationTextbox.Text + ".CameraSettings.txt"
                        os.rename(filename_settings, newfilename_settings)
                    except:
                        log("CameraSettings.txt file does not exist")
                    file = newname
                else:
                    log(file)

            if (self.checkbox_recon.Checked):
                flags = ''
                if (self.checkbox_showimage.Checked):
                    flags = flags + ' -d'
                if (self.checkbox_saveclahepng.Checked):
                    flags = flags + ' -c'
                if (self.checkbox_savefit.Checked):
                    flags = flags + ' -f'
                log("Flags: " + flags)
                # User to modify the path of example SER film, or to comment the next line for processing the capture film
                #file = "C:\\Temp\\10_28_39.ser"
                command_line = command_python + " " + str(flags) + " " + file
                log(command_line)
                os.system(command_line)

    def capture(self, sender, event):
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        if (SharpCap.SelectedCamera.Capturing == False):
            log("Start capture SER film")
            SharpCap.SelectedCamera.PrepareToCapture()
            SharpCap.SelectedCamera.RunCapture()
        else:
            log("Do nothing, the film is capturing")

    def stop(self, sender, event):
        global command_line
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        if (SharpCap.SelectedCamera.Capturing):
            log("Stop capture SER film")
            SharpCap.SelectedCamera.StopCapture()
            SharpCap.ShowNotification(None)

            Tel_Scan = "NA"

            Tel_Side = str(SharpCap.Mounts.SelectedMount.SideOfPier)
            if (Tel_Side == "WestSideLookingEast"):
                Tel_Side = "East"
            if (Tel_Side == "EastSideLookingWest"):
                Tel_Side = "West"

            if (self.checkbox_disc.Checked):
                Sun_Disc = "FullDisc"
            else:
                Sun_Disc = "PartialDisc"

            P, Bo = Carrington(datetime.datetime.utcnow())
            log(Tel_Scan + " , " + Tel_Side + " , " + Sun_Disc + " , P = " + P + " , Bo = " + Bo)

            file = SharpCap.GetLastCaptureFilename()

            f=open(file, "r+")
            f.seek(42)
            t_observer = self.InfObserverTextbox.Text
            t_observer = t_observer[:40]
            t_observer = t_observer.ljust(40)

            t_Instrument = Tel_Scan + " " + Tel_Side + " " + Sun_Disc + " " + self.TelRateTextbox.Text + " " + P + " " + Bo
            t_Instrument = t_Instrument[:40]
            t_Instrument = t_Instrument.ljust(40)

            t_Telescope = self.InfTelescopeTextbox.Text
            t_Telescope = t_Telescope[:40]
            t_Telescope = t_Telescope.ljust(40)

            f.write(t_observer)
            f.write(t_Instrument)
            f.write(t_Telescope)
            f.close()

            if (self.checkbox_rename.Checked):
                log("Rename SER file")
                current_time = datetime.datetime.utcnow()
                if (self.InformationTextbox.Text == ""):
                    newname = os.path.dirname(file) +"\\" + current_time.strftime('%Y%m%d%H%M%S') + Identificateur + ".ser"
                else:
                    newname = os.path.dirname(file) +"\\" + current_time.strftime('%Y%m%d%H%M%S') + Identificateur + "_" + self.InformationTextbox.Text + ".ser"
                log(newname)
                os.rename(file, newname)
                try:
                    filename_settings = os.path.splitext(file)[0] + ".CameraSettings.txt"
                    newfilename_settings = os.path.splitext(newname)[0] + ".CameraSettings.txt"
                    os.rename(filename_settings, newfilename_settings)
                except:
                    log("CameraSettings.txt file does not exist")
                file = newname
            else:
                if (self.InformationTextbox.Text != ""):
                    filename = os.path.splitext(file)[0]
                    newname = filename + "_" + self.InformationTextbox.Text + ".ser"
                    log(newname)
                    os.rename(file, newname)
                    try:
                        filename_settings = os.path.splitext(file)[0] + ".CameraSettings.txt"
                        newfilename_settings = os.path.splitext(file)[0] +  "_" + self.InformationTextbox.Text + ".CameraSettings.txt"
                        os.rename(filename_settings, newfilename_settings)
                    except:
                        log("CameraSettings.txt file does not exist")
                    file = newname
                else:
                    log(file)

            if (self.checkbox_recon.Checked):
                flags = ''
                if (self.checkbox_showimage.Checked):
                    flags = flags + ' -d'
                if (self.checkbox_saveclahepng.Checked):
                    flags = flags + ' -c'
                if (self.checkbox_savefit.Checked):
                    flags = flags + ' -f'
                log("Flags: " + flags)
                # User to modify the path of example SER film, or to comment the next line for processing the capture film
                #file = "C:\\Temp\\10_28_39.ser"
                command_line = command_python + " " + str(flags) + " " + file
                log(command_line)
                os.system(command_line)
        else:
            log("Do nothing")

    def exit(self, sender, event):
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        if (SharpCap.SelectedCamera.Capturing):
            log("Stop capture SER film")
            SharpCap.SelectedCamera.StopCapture()
            SharpCap.ShowNotification(None)
            file = SharpCap.GetLastCaptureFilename()
            log(file)
        if (SharpCap.Mounts.SelectedMount.Slewing):
            log("Stop telescope movement")
            tracking = SharpCap.Mounts.SelectedMount.Tracking
            SharpCap.Mounts.SelectedMount.Stop()
            SharpCap.Mounts.SelectedMount.Tracking = tracking
        log("Stop script spectrohéliographie et traitement avec INTI")
        self.Close()

    def framerate(self, sender, event):
        def log(text):
            self.process.log(time.strftime("%H:%M:%S") + " - " + text)

        self.FrameButton.BackColor = Color.Red
        CameraSettingFile = SharpCap.Settings.CreateCameraSettingsFile
        if (CameraSettingFile == False):
            SharpCap.Settings.CreateCameraSettingsFile = True
            SharpCap.Settings.Save()
        SharpCap.SelectedCamera.PrepareToCapture()
        SharpCap.SelectedCamera.RunCapture()
        time.sleep(2)
        SharpCap.SelectedCamera.StopCapture()
        SharpCap.ShowNotification(None)
        file = SharpCap.GetLastCaptureFilename()
        file_settings = os.path.splitext(file)[0] + ".CameraSettings.txt"
        with open(file_settings, 'r') as fp:
            for l_no, line in enumerate(fp):
                if "ActualFrameRate" in line:
                    break
        parts = line.split("=")[1]
        framerate = float(parts.split("f")[0])
        if (CameraSettingFile == False):
            SharpCap.Settings.CreateCameraSettingsFile = False
            SharpCap.Settings.Save()
        try:
            os.remove(file)
            os.remove(file_settings)
        except:
            print "No file to delete"

        self.FrameButton.BackColor = Color.Gainsboro

        now_JD = AstroUtils.AstroUtils().JulianDateUtc
        T = (now_JD - 2451545)/36525
        Lo = (0.0003032*T + 36000.76983)*T + 280.46645
        M = ((-0.00000048*T - 0.0001559)*T + 35999.05030)*T + 357.52910
        C = ((-0.000014*T - 0.004817)*T + 1.914600)*math.sin(math.radians(M))
        C = C +(-0.000101*T - 0.019993)*math.sin(math.radians(2*M)) + 0.000290*math.sin(math.radians(3*M))
        Theta = Lo + C
        Epsilon = ((0.001813*T - 0.00059)*T - 46.815)*T
        Epsilon = 23.439291 + Epsilon/3600
        Sun_Delta = math.asin(math.sin(math.radians(Epsilon)) * math.sin(math.radians(Theta)))  # Delta coordinate of the sun at the "now date"

        Binning_Liste = SharpCap.SelectedCamera.Controls.Binning.AvailableValues                # Available binning (1x1, 2x2, ...)
        binning = Binning_Liste[SharpCap.SelectedCamera.Controls.Binning.Bin - 1]               # Selected binning
        N_pixel = 2000 * math.tan(math.radians(0.2666667)) * float(self.TelFocaleTextbox.Text) * math.cos(Sun_Delta)
        N_pixel = (N_pixel * 125 / 80) / SharpCap.SelectedCamera.PixelSize.Height
        Scan_Time = N_pixel / framerate
        Tel_scan_rate = 128 / Scan_Time
        log("Binning = " + binning)
        log("Pixel size = " + str(SharpCap.SelectedCamera.PixelSize.Height) + " µm")
        log("Frame rate = " + str(round(framerate,1)) + " fps")
        log("Sun size = " + str(round(N_pixel,1)) + " pixel")
        log("Sun delta = " + str(round(math.degrees(Sun_Delta),1)) + " deg")
        log("Telescope scan rate = " + str(round(Tel_scan_rate,1)) + " x")

def launch_form():
    form = SHG_CaptureMenuForm()
    form.icon = Icon(default_icon)
    form.StartPosition = FormStartPosition.CenterScreen
    form.TopMost = True
    form.Show()

SharpCap.AddCustomButton("SHG__Capture |", Image.FromFile(default_icon), "SHG Capture", launch_form)


